package Prog1Tools;

import java.io.*;
import java.util.StringTokenizer;

/**
 * Klasse zum einfachen Umgang mit CSV-Dateien.
 *
 * @author hagen.buchwald@kit.edu
 * @version 1.00
 * @date 15.08.2010
 */
public class CSVFile implements CSVFileSpec
{
    /** Konstante, die den Abstandshalter zwischen den Eintraegen der CSV-CSV-Datei definiert. */
    public static String DELIMITER = ";";
    
    private String name;
    
    /**
     * Kommando zum Erzeugen einer neuen Instanz der Klasse CSV-Datei.
     */
    public static CSVFile getInstance()
    {
        CSVFile result;
        result = new CSVFile();
        return result;
    }
    
    public void setName (String name)
    {
        this.name = name;
    }
    
    public String getName ()
    {
        String result;
        result = name;
        return result;
    }
    
    public void setContent (String[][] content)
    {
        PrintWriter output = null;
        try
        {
            output = new PrintWriter(new FileWriter(new File(name)));
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        for (int row = 0; row < content.length; row++)
        {
            String zeile = content[row][0];
            for (int col = 1; col < content[row].length; col++)
            {
                zeile = zeile + DELIMITER + content[row][col];
            }
            output.println(zeile);
        }
        output.close();
    }
    
    public String[][] getContent ()
    {
        String[][] result = null;
        BufferedReader input = null;
        StringTokenizer st;
        int zeilen = 0;
        int spalten = 0;
        int row;
        int col;
        String zeile;
        String token;
        
        // Anzahl Zeilen und Spalten ermitteln
        try
        {
            input = new BufferedReader(new FileReader(new File(name)));
            zeilen = 0;
            spalten = 0;
            zeile = input.readLine();
            if (zeile != null)
            {
                st = new StringTokenizer(zeile, DELIMITER);
                while (st.hasMoreTokens())
                {
                    spalten = spalten + 1;
                    token = st.nextToken();
                }
                while (zeile != null)
                {
                    zeilen = zeilen + 1;
                    zeile = input.readLine();
                }
            }
            input.close();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        
        // CSV-Dateiinhalt auslesen
        if (zeilen > 0 && spalten > 0)
        {
            result = new String[zeilen][spalten];
            try
            {
                input = new BufferedReader(new FileReader(new File(name)));
                row = 0;
                zeile = input.readLine();
                while (zeile != null)
                {
                    col = 0;
                    st = new StringTokenizer(zeile, DELIMITER);
                    while (st.hasMoreTokens())
                    {
                        token = st.nextToken();
                        result[row][col] = token;
                        col = col + 1;
                    }
                    zeile = input.readLine();
                    row = row + 1; 
                }
                input.close();
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
        }
        
        return result;
    }
    
    public void create ()
    {
        File file = new File(name);
        if (file.exists())
        {
            file.delete();
        }
        try
        {
            file.createNewFile();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
    }
    
    public void delete ()
    {
        File file = new File(name);
        file.delete();
    }
    
    public boolean exists ()
    {
        boolean result = false;
        File file = new File(name);
        result = file.exists();
        return result;
    }
    
    public boolean equals (CSVFileSpec other)
    {
        boolean result = true;
        String[][] thisContent = getContent();
        String[][] otherContent = other.getContent();
        
        if (thisContent == null && otherContent == null)
        {
            result = true;
        }
        else if (thisContent == null && otherContent != null)
        {
            result = false;
        }
        else if (thisContent != null && otherContent == null)
        {
            result = false;
        }
        else
        {
            // number of rows must be equal
            result = result && (thisContent.length == otherContent.length);
            
            // number of columns must be equal
            result = result && (thisContent[0].length== otherContent[0].length);
            
            // all items must be equal
            for (int row = 0; row < thisContent.length; row++)
            {
                for (int col = 0; col < thisContent[row].length; col++)
                {
                    result = result && (thisContent[row][col].equals(otherContent[row][col]));
                }
            }
        }
        
        return result;
    }
}
