package Prog1Tools;

import net.sourceforge.c4j.*;

/**
 * Spezifikation einer Klasse zum einfachen Umgang mit CSV-Dateien.
 *
 * @author hagen.buchwald@kit.edu
 * @version 1.00
 * @date 15.08.2010
 */
@ContractReference (contractClassName = "CSVFileSpecContract")
public interface CSVFileSpec
{
    /**
     * Kommando zum Setzen des Namens der CSV-Datei.
     *
     * @param name Name der CSV-Datei
     */
    public void setName (String name);
    
    /**
     * Abfrage des Namens der CSV-Datei.
     *
     * @return Name der CSV-Datei
     */
    @Pure
    public String getName ();
    
    /**
     * Kommando zum Schreiben des durch content definierten Inhalts in die CSV-Datei.
     *
     * @param content zweidimensionales String-Feld
     */
    public void setContent (String[][] content);
    
    /**
     * Abfrage des Inhalts der CSV-Datei.
     *
     * @return Inhalt der CSV-Datei
     */
    @Pure
    public String[][] getContent ();
    
    /**
     * Kommando zum Erzeugen einer leeren CSV-Datei.
     */
    public void create ();
    
    /**
     * Kommando zum Loeschen der CSV-Datei.
     */
    public void delete ();
    
    /**
     * Abfrage, ob die CSV-Datei existiert, d.h. bereits erzeugt wurde.
     *
     * @return true, wenn die CSV-Datei existiert; false sonst
     */
    @Pure
    public boolean exists ();
    
    /**
     * Abfrage, ob der Inhalt dieser CSV-Datei gleich dem Inhalt der CSV-Datei other ist.
     *
     * @param other CSV-Datei, mit deren Inhalt der Inhalt dieser CSV-Datei verglichen werden soll
     * @return true, wenn der Inhalt beider CSV-Dateien gleich ist; false sonst
     */
    @Pure
    public boolean equals (CSVFileSpec other);
}
    
