package Prog1Tools;

import net.sourceforge.c4j.*;

/**
 * Vertragsklasse des Interfaces CSVFileSpec.
 *
 * @author hagen.buchwald@kit.edu
 * @version 1.00
 * @date 15.08.2010
 */
public class CSVFileSpecContract extends ContractBase<CSVFileSpec>
{
    public CSVFileSpecContract (CSVFileSpec target)
    {
        super (target);
    }
    
    public void pre_setName (String name)
    {
        assert name != null : "name_not_null";
    }
    
    public void post_setName (String name)
    {
        assert name.equals(m_target.getName()) : "name_set";
    }
    
    public void pre_getName ()
    {
        
    }
    
    public void post_getName ()
    {
        
    }
    
    public void pre_setContent (String[][] content)
    {
        assert m_target.getName() != null : "name_set";
        assert content != null : "content_not_null";
        assert content.length > 0 : "content_has_at_least_one_row";
        assert content[0].length > 0 : "content_has_at_least_one_column";
        
        boolean allRowsSameLength = true;
        for (int row = 1; row < content.length; row++)
        {
            allRowsSameLength = allRowsSameLength && (content[0].length == content[row].length);
        }
        assert allRowsSameLength : "all_rows_same_length";
        
        boolean allItemsNotNull = true;
        for (int row = 0; row < content.length; row++)
        {
            for (int col = 0; col < content[0].length; col++)
            {
                allItemsNotNull = allItemsNotNull && (content[row][col] != null);
            }
        }
        assert allItemsNotNull : "all_items_not_null";
    }
    
    public void post_setContent (String[][] content)
    {
        boolean allContentItemsSet = true;
        String[][] thisContent = m_target.getContent();
        for (int row = 0; row < content.length; row++)
        {
            for (int col = 0; col < content[0].length; col++)
            {
                allContentItemsSet = allContentItemsSet && (content[row][col].equals(thisContent[row][col]));
            }
        }
        assert allContentItemsSet : "all_content_items_set";
    }
    
    public void pre_getContent ()
    {
        assert m_target.getName() != null : "name_set";
    }
    
    public void post_getContent ()
    {
         
    }
    
    public void pre_create ()
    {
        assert m_target.getName() != null : "name_set"; 
    }
    
    public void post_create ()
    {
        
    }
    
    public void pre_delete ()
    {
        assert m_target.getName() != null : "name_set";
        assert m_target.exists() : "exists";
    }
    
    public void post_delete ()
    {
        assert !m_target.exists() : "not_exists";
    }
    
    public void pre_exists ()
    {
        assert m_target.getName() != null : "name_set";
    }
    
    public void post_exists ()
    {
        
    }
    
    public void pre_equals (CSVFileSpec other)
    {
        assert m_target.getName() != null : "this_name_set";
        assert m_target.exists() : "this_exists";
        assert other.getName() != null : "other_name_set";
        assert other.exists() : "other_exists";
    }
    
    public void post_equals (CSVFileSpec other)
    {
        boolean result = (Boolean) getReturnValue();
        String[][] thisContent = m_target.getContent();
        String[][] otherContent = other.getContent();
        boolean expected = true;
        
        if (thisContent == null && otherContent == null)
        {
            expected = true;
        }
        else if (thisContent == null && otherContent != null)
        {
            expected = false;
        }
        else if (thisContent != null && otherContent == null)
        {
            expected = false;
        }
        else
        {
            // number of rows must be equal
            expected = expected && (thisContent.length == otherContent.length);
            
            // number of columns must be equal
            expected = expected && (thisContent[0].length== otherContent[0].length);
            
            // all items must be equal
            for (int row = 0; row < thisContent.length; row++)
            {
                for (int col = 0; col < thisContent[row].length; col++)
                {
                    expected = expected && (thisContent[row][col].equals(otherContent[row][col]));
                }
            }
        }
        
        assert result == expected : "expected_result";
    }
    
}
