package Prog1Tools;

import net.sourceforge.c4j.*;

/**
 * Spezifikation eines Plotters zur Darstellung von Funktionsgraphen.
 *
 * @version 1.0
 * @author hagen.buchwald@kit.edu
 * @date 2010-09-06
 */
@ContractReference (contractClassName = "FunctionPlotterSpecContract")
public interface FunctionPlotterSpec
{
    /** Abstand des Koordinatensystems zum Bildschirmrand in Pixeln. */
    public final int MARGIN = 20;
    
    /** Verschiebung der x-Position des Textes zur Achsenbeschriftung. */
    public final int X_OFFSET = -5;
    
    /** Verschiebung der y-Position des Textes zur Achsenbeschriftung. */
    public final int Y_OFFSET = -5;
    
    /**
     * Kommando zum Setzen des Wertebereichs, der von der x-Achse abgedeckt werden soll.
     * 
     * @param leftY linker Rand des Wertebereichs
     * @param rightX rechter Rand des Wertebereichs
     */
    public void setXRange (double leftX, double rightX);
    
    /**
     * Kommando zum Setzen des Wertebereichs, der von der y-Achse abgedeckt werden soll.
     * 
     * @param lowerY unterer Rand des Wertebereichs
     * @param upperY oberer Rand des Wertebereichs
     */
    public void setYRange (double lowerY, double upperY);
    
    /**
     * Abfrage des linken Rands des Wertebereichs, der von der x-Achse abgedeckt wird.
     * 
     * @param linker Rand des Wertebereichs
     */
    @Pure
    public double getLeftX ();
    
    /**
     * Abfrage des rechten Rands des Wertebereichs, der von der x-Achse abgedeckt wird.
     * 
     * @param rechter Rand des Wertebereichs
     */
    @Pure
    public double getRightX ();
    
    /**
     * Abfrage des unteren Rands des Wertebereichs, der von der y-Achse abgedeckt wird.
     * 
     * @param unterer Rand des Wertebereichs
     */
    @Pure
    public double getLowerY ();
    
    /**
     * Abfrage des oberen Rands des Wertebereichs, der von der y-Achse abgedeckt wird.
     * 
     * @param oberer Rand des Wertebereichs
     */
    @Pure
    public double getUpperY ();
    
    /**
     * Kommando zum Zeichnen der x-Achse (Abszisse).
     * Der Wert x = 0 liegt genau im Ursprung.
     */
    public void drawAbscissae ();
    
    /**
     * Kommando zum Zeichnen der y-Achse (Ordinate).
     * Der Wert y = 0 liegt genau im Ursprung.
     */
    public void drawOrdinate ();
    
    /**
     * Kommando zum Zeichnen des Koordinaten-Systems.
     * Der Nullpunkt des Koordinatensystems liegt im Ursprung des Screens.
     */
    public void drawCoordinateSystem ();
    
    /**
     * Kommando zum Zeichnen eines Funktionsgraphen im Wertebereich anhand einer Wertetabelle.
     * 
     * @param xValues x-Werte der Wertetabelle
     * @param yPoints zugehoerige y-Werte (Funktionswerte y = f(x)) der Wertetabelle
     * @param isFilled bei false wird nur die Linie des Graphen in der aktuellen Farbe gezeichnet;
     * bei true wird zusaetzlich der Bereich zwischen x-Achse und Graph mit der aktuellen Zeichenfarbe gefüllt.
     */
    public void drawGraph (double[] xValues, double[] yValues, boolean isFilled);
    
    /**
     * Kommando zum Zeichnen des Graphen der uebergebenen Funktion f(x) im Wertebereich.
     * 
     * @param f Funktion f(x)
     * @param isFilled bei false wird nur die Linie des Graphen in der aktuellen Farbe gezeichnet;
     * bei true wird zusaetzlich der Bereich zwischen x-Achse und Graph mit der aktuellen Zeichenfarbe gefüllt.
     */
    public void drawGraph (FunctionSpec f, boolean isFilled);
    
}
    
