package Prog1Tools;

import java.awt.*;

/** 
 * Implementierung des Grafikelements Ovalbogen.
 * 
 * @author  hagen.buchwald@kit.edu
 * @version 1.1
 * @date 2011-03-23
 */
public class GraphicArc extends GraphicItem
{
    private int x;
    private int y;
    private int width;
    private int height;
    private int startAngle;
    private int arcAngle;
    private Color color;
    private boolean isFilled;
    
    /**
     * Konstruktor des Grafikelements Ovalbogen.
     * 
     * @param x x-Koordinate der oberen linken Ecke des gedachten Rechtecks, das das zu zeichnende Oval umgibt
     * @param y y-Koordinate der oberen linken Ecke des gedachten Rechtecks, das das zu zeichnende Oval umgibt
     * @param width Breite des gedachten Rechtecks, das das zu zeichnende Oval umgibt
     * @param height Hoehe des gedachten Rechtecks, das das zu zeichnende Oval umgibt
     * @param startAngle Winkel (0 .. 360 Grad), ab dem der Ovalbogen gezeichnet werden soll, wobei 0 Grad der 3 Uhr Position entspricht.
     * @param arcAngle Winkel (0 .. 360 Grad), den der gezeichnete Ovalbogen besitzen soll, wobei sich der Ovalbogen im Gegenuhrzeigersinn aufspannt.
     * @param color Farbe, in der der Ovalbogen gezeichnet werden soll
     * @param isFilled bei false wird nur der Linienzug des Ovalbogens gezeichnet;
     * bei true wird die Flaeche des Ovalbogens in der Zeichenfarbe eingefaerbt
     */
    public GraphicArc (int x, int y, int width, int height, int startAngle, int arcAngle, Color color, boolean isFilled)
    {
        this.x = x;
        this.y = y;
        this.width = width;
        this.height = height;
        this.startAngle = startAngle;
        this.arcAngle = arcAngle;
        this.color = color;
        this.isFilled = isFilled;
    }
    
    public void paint (Graphics graphics)
    {
        Color oldColor = graphics.getColor();
        graphics.setColor(color);
        if (!isFilled)
        {
            graphics.drawArc(x, y, width, height, startAngle, arcAngle);
        }
        else
        {
            graphics.fillArc(x, y, width, height, startAngle, arcAngle);
        }
        graphics.setColor(oldColor);
    }

    public void move (int xDistance, int yDistance)
    {
    	x = x + xDistance;
    	y = y + yDistance;
    }
}
    
