package Prog1Tools;

import java.awt.*;

/** 
 * Implementierung des Grafikelements Rechteck mit abgerundeten Ecken.
 * 
 * @author  hagen.buchwald@kit.edu
 * @version 1.1
 * @date 2011-03-23
 */
public class GraphicRectangleRounded extends GraphicItem
{
    private int x;
    private int y;
    private int width;
    private int height;
    private int arcWidth;
    private int arcHeight;
    private Color color;
    private boolean isFilled;
    
    /**
     * Konstruktor des Grafikelements Rechteck mit abgerundeten Ecken.
     * 
     * @param x x-Koordinate der oberen linken Ecke des Rechtecks
     * @param y y-Koordinate der oberen linken Ecke des Rechtecks
     * @param width Breite des Rechtecks
     * @param height Hoehe des Rechtecks
     * @param arcWidth horizontaler Radius des Ovals, das die Kruemmung der abgerundeten Ecken des Rechtecks beschreibt
     * @param arcHeight vertikaler Radius des Ovals, das die Kruemmung der abgerundeten Ecken des Rechtecks beschreibt
     * @param color Farbe, in der das Rechteck gezeichnet werden soll
     * @param isFilled bei false wird nur der Linienzug des Rechtecks gezeichnet;
     * bei true wird die Flaeche des Rechtecks in der Zeichenfarbe eingefaerbt
     */
    public GraphicRectangleRounded (int x, int y, int width, int height, int arcWidth, int arcHeight, Color color, boolean isFilled)
    {
        this.x = x;
        this.y = y;
        this.width = width;
        this.height = height;
        this.arcWidth = arcWidth;
        this.arcHeight = arcHeight;
        this.color = color;
        this.isFilled = isFilled;
    }
    
    public void paint (Graphics graphics)
    {
        Color oldColor = graphics.getColor();
        graphics.setColor(color);
        if (!isFilled)
        {
            graphics.drawRoundRect(x, y, width, height, arcWidth, arcHeight);
        }
        else
        {
            graphics.fillRoundRect(x, y, width, height, arcWidth, arcHeight);
        }
        graphics.setColor(oldColor);
    }

    public void move (int xDistance, int yDistance)
    {
    	x = x + xDistance;
    	y = y + yDistance;
    }
    
}
    
