package Prog1Tools;

import net.sourceforge.c4j.*;
import java.awt.*;

/** 
 * Spezifikation eines grafikfaehigen Farbbildschirms.
 * 
 * @author  hagen.buchwald@kit.edu
 * @version 1.11
 * @date 2011-04-24
 */
@ContractReference (contractClassName = "GraphicScreenSpecContract")
public interface GraphicScreenSpec
{
    /**
     * Abfrage der zum Zeichnen verwendeten Farbe.
     */
    @Pure
    public Color getColor ();
    
    /**
     * Kommando zum Setzen der zum Zeichnen verwendeten Farbe.
     *
     * @param color Farbe, die zum Zeichnen verwendet werden soll
     */
    public void setColor (Color color);
    
    /**
     * Kommando zum Verschieben des Ursprungs von der urspruenglichen
     * Position (0, 0) (obere, linke Ecke des GraphicScreens) an die
     * Position (x, y).
     *
     * @param x x-Koordinate der neuen Ursprungs-Position
     * @param y y-Koordinate der neuen Ursprungs-Position
     */
    public void setOrigin (int x, int y);
    
    /**
     * Abfrage der Koordinaten des Nullpunkt des GraphicScreens.
     *
     * @return Nullpunkt des GraphicScreens
     */
    public Point getOrigin ();
    
    /**
     * Kommando zum Einzeichnen des Ursprungs in Form eines Fadenkreuzes.
     */
    public void drawOrigin ();
    
    /**
     * Kommando zum Zeichnen einer Linie.
     *
     * @param x1 x-Koordinate des Startpunktes der Linie
     * @param y1 y-Koordinate des Startpunktes der Linie
     * @param x2 x-Koordinate des Endpunktes der Linie
     * @param y2 y-Koordinate des Endpunktes der Linie
     */
    public void drawLine (int x1, int y1, int x2, int y2);
    
    /**
     * Kommando zum Zeichnen einer Polylinie.
     *
     * @param xPoints x-Koordinaten aller Punkte der Polylinie
     * @param yPoints y-Koordinaten aller Punkte der Polylinie
     * @param nPoints Anzahl der Punkte, bis zu der die Polylinie gezeichnet werden soll
     */
    public void drawPolyline (int[] xPoints, int[] yPoints, int nPoints);
    
    /**
     * Kommando zum Zeichnen eines Polygons.
     *
     * @param xPoints x-Koordinaten aller Punkte des Polygons
     * @param yPoints y-Koordinaten aller Punkte des Polygons
     * @param nPoints Anzahl der Punkte, bis zu der das Polygon gezeichnet werden soll
     * @param isFilled bei false wird nur der Linienzug des Polygons gezeichnet;
     * bei true wird die Flaeche des Polygons in der Zeichenfarbe eingefaerbt
     */
    public void drawPolygon (int[] xPoints, int[] yPoints, int nPoints, boolean isFilled);
    
    /**
     * Kommando zum Zeichnen eines Texts.
     * Bezugspunkt ist die linke untere Ecke eines gedachten Rechtecks,
     * das den Text umgibt.
     *
     * @param x x-Koordinate des Bezugspunkts des Texts
     * @param y y-Koordinate des Bezugspunkts des Texts
     * @param text Text, der dargestellt werden soll
     */
    public void drawText (int x, int y, String text);
    
    /**
     * Kommando zum Zeichnen eines Kreises.
     *
     * @param x x-Koordinate des Kreismittelpunkts
     * @param y y-Koordinate des Kreismittelpunkts
     * @param radius Radius des Kreises
     * @param isFilled bei false wird nur der Linienzug des Kreises gezeichnet;
     * bei true wird die Flaeche des Kreises in der Zeichenfarbe eingefaerbt
     */
    public void drawCircle (int x, int y, int radius, boolean isFilled);
    
    /**
     * Kommando zum Zeichnen eines Ovals.
     *
     * @param x x-Koordinate der oberen linken Ecke des gedachten Rechtecks, das das zu zeichnende Oval umgibt
     * @param y y-Koordinate der oberen linken Ecke des gedachten Rechtecks, das das zu zeichnende Oval umgibt
     * @param width Breite des gedachten Rechtecks, das das zu zeichnende Oval umgibt
     * @param height Hoehe des gedachten Rechtecks, das das zu zeichnende Oval umgibt
     * @param isFilled bei false wird nur der Linienzug des Ovals gezeichnet;
     * bei true wird die Flaeche des Ovals in der Zeichenfarbe eingefaerbt
     */
    public void drawOval (int x, int y, int width, int height, boolean isFilled);
    
    /**
     * Kommando zum Zeichnen eines Rechtecks.
     *
     * @param x x-Koordinate der oberen linken Ecke des Rechtecks
     * @param y y-Koordinate der oberen linken Ecke des Rechtecks
     * @param width Breite des Rechtecks
     * @param height Hoehe des Rechtecks
     * @param isFilled bei false wird nur der Linienzug des Rechtecks gezeichnet;
     * bei true wird die Flaeche des Rechtecks in der Zeichenfarbe eingefaerbt
     */
    public void drawRectangle (int x, int y, int width, int height, boolean isFilled);
    
    /**
     * Kommando zum Zeichnen eines eingestanzten bzw. erhobenen Rechtecks.
     *
     * @param x x-Koordinate der oberen linken Ecke des Rechtecks
     * @param y y-Koordinate der oberen linken Ecke des Rechtecks
     * @param width Breite des Rechtecks
     * @param height Hoehe des Rechtecks
     * @param isFilled bei false wird nur der Linienzug des Rechtecks gezeichnet;
     * bei true wird die Flaeche des Rechtecks in der Zeichenfarbe eingefaerbt
     * @param isRaised bei false wird das Rechteck eingestanzt gezeichnet;
     * bei true wird das Rechteck erhoben gezeichnet
     */
    public void drawRectangle3D (int x, int y, int width, int height, boolean isFilled, boolean isRaised);
    
    /**
     * Kommando zum Zeichnen eines Rechtecks mit abgerundeten Ecken.
     *
     * @param x x-Koordinate der oberen linken Ecke des Rechtecks
     * @param y y-Koordinate der oberen linken Ecke des Rechtecks
     * @param width Breite des Rechtecks
     * @param height Hoehe des Rechtecks
     * @param arcWidth horizontaler Radius des Ovals, das die Kruemmung der abgerundeten Ecken des Rechtecks beschreibt
     * @param arcHeight vertikaler Radius des Ovals, das die Kruemmung der abgerundeten Ecken des Rechtecks beschreibt
     * @param isFilled bei false wird nur der Linienzug des Rechtecks gezeichnet;
     * bei true wird die Flaeche des Rechtecks in der Zeichenfarbe eingefaerbt
     */
    public void drawRectangleRounded (int x, int y, int width, int height, int arcWidth, int arcHeight, boolean isFilled);
    
    /**
     * Kommando zum Zeichnen eines Ovalbogens.
     *
     * @param x x-Koordinate der oberen linken Ecke des gedachten Rechtecks, das das zu zeichnende Oval umgibt
     * @param y y-Koordinate der oberen linken Ecke des gedachten Rechtecks, das das zu zeichnende Oval umgibt
     * @param width Breite des gedachten Rechtecks, das das zu zeichnende Oval umgibt
     * @param height Hoehe des gedachten Rechtecks, das das zu zeichnende Oval umgibt
     * @param startAngle Winkel (0 .. 360 Grad), ab dem der Ovalbogen gezeichnet werden soll, wobei 0 Grad der 3 Uhr Position entspricht.
     * @param arcAngle Winkel (0 .. 360 Grad), den der gezeichnete Ovalbogen besitzen soll, wobei sich der Ovalbogen im Gegenuhrzeigersinn aufspannt.
     * @param isFilled bei false wird nur der Linienzug des Ovalbogens gezeichnet;
     * bei true wird die Flaeche des Ovalbogens in der Zeichenfarbe eingefaerbt
     */
    public void drawArc (int x, int y, int width, int height, int startAngle, int arcAngle, boolean isFilled);
    
    /**
     * Abfrage, wie viele Grafikobjekte auf dem Grafikbildschirm liegen.
     *
     * @return Anzahl der Grafikobjekte, die auf dem Grafikbildschirm liegen
     */
    @Pure
    public int getNumberOfItems ();
    
    /**
     * Abfrage des maximal zulaessigen Wertes fuer die x-Koordinate eines Punkts, so dass dieser gerade noch im Anzeigebereich des Grafikbildschirms liegt.
     *
     * @return maximal zulaessiger Wert fuer die x-Koordinate eines Punkte
     */
    @Pure
    public int getMaxX ();
    
    /**
     * Abfrage des maximal zulaessigen Wertes fuer die y-Koordinate eines Punkts, so dass dieser gerade noch im Anzeigebereich des Grafikbildschirms liegt.
     *
     * @return maximal zulaessiger Wert fuer die y-Koordinate eines Punkte
     */
    @Pure
    public int getMaxY ();
    
    /**
     * Abfrage der maximalen Anzahl der Grafikobjekte, die auf dem Grafikbildschirm gezeichnet werden koennen.
     *
     * @return maximal zulaessiger Wert fuer die Anzahl der Grafikobjekte auf dem Grafikbildschirm
     */
    @Pure
    public int getMaxNumberOfItems ();

    /**
     * Abfrage eines Grafikobjekts.
     *
     * @param index Index des Grafikobjekts.
     * @return Grafikobjekt mit dem angegebenen Index
     */
    @Pure
    public GraphicItem getItem (int index);
}
    
