package Prog1Tools;

/** 
 * Klasse zum Erzeugen von Permutationen.
 * 
 * @author  hagen.buchwald@kit.edu
 * @version 1.0
 * @date 2010-08-30
 */
public class Permutation implements PermutationSpec
{
    
    private int[] a;        // Feld der Indizes, die permutiert werden.
    private long numLeft;   // Anzahl der Permutationen, die noch ausstehen.
    private long total;     // Anzahl der Permutationen insgesamt.
    
    /**
     * Konstruktor.
     *
     * @param n Laenge des Indizes-Feldes a, das erzeugt und permutiert werden soll.
     */
    public Permutation (int n)
    {
        a = new int[n];
        total = getFactorial (n);
        reset ();
    }
    
    public void reset ()
    {
        for (int i = 0; i < a.length; i++) {
            a[i] = i;
        }
        numLeft = total;
    }
    
    public long getNumLeft ()
    {
        return numLeft;
    }
    
    public long getTotal ()
    {
        return total;
    }
    
    public boolean hasMore ()
    {
        return numLeft != 0;
    }
    
    public long getFactorial (int n)
    {
        long result = 1;
        for (int i = n; i > 1; i--) {
            result = result * i;
        }
        return result;
    }
    
    public int[] getNext ()
    {
        int[] result = null;
        
        if (numLeft == total) {
            numLeft = numLeft - 1;
        }
        else
        {
            int temp;
            
            // Find largest index j with a[j] < a[j+1]
            
            int j = a.length - 2;
            while (a[j] > a[j+1]) {
                j--;
            }
            
            // Find index k such that a[k] is smallest integer
            // greater than a[j] to the right of a[j]
            
            int k = a.length - 1;
            while (a[j] > a[k]) {
                k--;
            }
            
            // Interchange a[j] and a[k]
            
            temp = a[k];
            a[k] = a[j];
            a[j] = temp;
            
            // Put tail end of permutation after jth position in increasing order
            
            int r = a.length - 1;
            int s = j + 1;
            
            while (r > s) {
                temp = a[s];
                a[s] = a[r];
                a[r] = temp;
                r--;
                s++;
            }
            
            numLeft = numLeft - 1;
            
        }
        result = a;
        return result;
    }
    
}

