/*
    Copyright (C) 2010  (Hagen Buchwald)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package Prog1Tools;

import java.awt.*;
import java.awt.event.*;
import javax.sound.midi.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.io.*;
import java.util.*;

/** 
 * Die abstrakte Klasse Screen definiert Standards wie die KIT-Farben
 * und stellt grundlegende Funktionen zum Umgang mit einem Screen
 * zur Verfuegung.
 * 
 * @author  hagen.buchwald@kit.edu
 * @version 2.0
 * @date 2010-09-08
 */
@SuppressWarnings("serial")
public abstract class Screen extends JFrame
                             implements ScreenSpec
{
    protected int selectedMouseButton = 0;
    protected int selectedKeyCode = 0;
    protected char selectedKeyChar = 0;
    protected int selectedOption = CANCEL_OPTION;
    protected Object selectedSource = null;
    protected String fontName = FONT_NAME;
    protected int fontStyle = FONT_STYLE;
    protected int fontSize = FONT_SIZE;
    
    protected MouseListener mouseListener = new MouseListener();
    protected KeyListener keyListener = new KeyListener();
    
    protected static Sequencer player = null;
    protected static Sequence seq = null;
    protected static Track track = null;
    
    protected Color choosenColor = BLACK;
    protected File choosenFile = null;
    protected JFileChooser fileChooser = null;
    protected FileNameExtensionFilter fileFilter = new FileNameExtensionFilter("CSV Dateien", "csv");
        
    protected boolean debug;
    
    
    public int getSelectedMouseButton ()
    {
        int result = ScreenSpec.MOUSEBUTTON_RESET_VALUE;
        result = selectedMouseButton;
        return result;
    }
    
    public int getSelectedKeyCode ()
    {
        int result = ScreenSpec.KEYCODE_RESET_VALUE;
        result = selectedKeyCode;
        return result;
    }
    
    public char getSelectedKeyChar ()
    {
        char result = ScreenSpec.KEYCHAR_RESET_VALUE;
        result = selectedKeyChar;
        return result;
    }
    
    public int getSelectedOption ()
    {
        int result = 0;
        result = selectedOption;
        return result;
    }
    
    public Color getChoosenColor ()
    {
        Color result = null;
        result = choosenColor;
        return result;
    }
    
    public File getChoosenFile ()
    {
        File result = null;
        result = choosenFile;
        return result;
    }
    
    public void setDebug (boolean debug)
    {
        this.debug = debug;
    }
    
    public void setResizable (boolean resizable)
    {
        super.setResizable(resizable);
    }
    
    public void resetFont ()
    {
        fontName = FONT_NAME;
        fontStyle = FONT_STYLE;
        fontSize = FONT_SIZE;
    }
    
    public void setFont (String fontName, int fontStyle, int fontSize)
    {
        this.fontName = fontName;
        this.fontStyle = fontStyle;
        this.fontSize = fontSize;
    }
    
    public void clearScreen ()
    {
        clearScreen(true);
    }
    
    public abstract void clearScreen (boolean fastMode);
    
    public static void playSound (int instrument, int note, int duration)
    {
        boolean isValidInstrument = (0 <= instrument) && (instrument <= 127);
        boolean isValidNote = (0 <= note) && (note <= 127);
        boolean isValidDuration = (0 <= duration);
        final int playerInstrument = instrument;
        final int playerNote = note;
        final int playerDuration = duration;
        
        if (isValidInstrument && isValidNote && isValidDuration)
        {
            try
            {
                Runnable playTune = new Runnable()
                {
                    public void run()
                    {
                        try
                        {
                            player = MidiSystem.getSequencer();
                            seq = new Sequence(Sequence.PPQ, 4);
                            track = seq.createTrack();
            
                            ShortMessage first = new ShortMessage();
                            first.setMessage(192, 1, playerInstrument, 0);
                            MidiEvent instrumentWechsel = new MidiEvent(first, 1);
                            track.add(instrumentWechsel);
                
                            ShortMessage a = new ShortMessage();
                            a.setMessage(144, 1, playerNote, 100);
                            MidiEvent noteOn = new MidiEvent(a, 1);
                            track.add(noteOn);
                
                            ShortMessage b = new ShortMessage();
                            b.setMessage(128, 1, playerNote, 100);
                            MidiEvent noteOff = new MidiEvent(b, 16);
                            track.add(noteOff);
                
                            player.open();
                            player.setSequence(seq);
                        }
                        catch (Exception e)
                        {
                            e.printStackTrace();
                        }
                        player.start();
                        try
                        {
                            Thread.sleep(playerDuration);
                        }
                        catch (Exception e)
                        {
                            e.printStackTrace();
                        }
                        player.close();
                    }
                };
                try
                {
                    SwingUtilities.invokeLater(playTune);
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
         
    } 
    
    public static void pause (int duration)
    {
        try
        {
            Thread.sleep(duration);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
    
    public void showMessageDialog (String title, String message)
    {
        JOptionPane.showMessageDialog(this, message, title, JOptionPane.INFORMATION_MESSAGE);
        selectedOption = OK_OPTION;
    }
    
    public int showOptionDialogAbbruchOK (String title, String message)
    {
        int result = CLOSED_OPTION;
        Object[] options = {"OK", "Abbruch" };
        
        result = JOptionPane.showOptionDialog
            (
                this,
                message,
                title,
                JOptionPane.YES_NO_CANCEL_OPTION,
                JOptionPane.QUESTION_MESSAGE,
                null,
                options,
                options[0]
            );
        
        if (result == 0)
        {
            selectedOption = OK_OPTION;
        }
        else
        {
            selectedOption = CANCEL_OPTION;
        }
        
        return result;
    }
    
    public int showOptionDialogJaNein (String title, String message)
    {
        int result = CLOSED_OPTION;
        Object[] options = {"Ja", "Nein" };
        
        result = JOptionPane.showOptionDialog
            (
                this,
                message,
                title,
                JOptionPane.YES_NO_CANCEL_OPTION,
                JOptionPane.QUESTION_MESSAGE,
                null,
                options,
                options[0]
            );
        
        return result;
    }
    
    public String showInputDialogComboBox (String title, String message, String[] comboBoxItems)
    {
        String result = null;
        boolean dialogSchliessen = false;
        String leereAuswahl = null;
        String leererString = "";
        
        if (comboBoxItems != null)
        {
            leereAuswahl = comboBoxItems[0];
        }
        
        selectedOption = OK_OPTION;
        
        while (!dialogSchliessen)
        {
            result = (String) JOptionPane.showInputDialog
                (
                    this,
                    message,
                    title,
                    JOptionPane.PLAIN_MESSAGE,
                    null,
                    comboBoxItems,
                    leereAuswahl
                );
            
            if (result == null)
            {
                // Abbrechen-Button wurde gedrueckt
                dialogSchliessen = true;
                selectedOption = CANCEL_OPTION;
            }
            else
            {
                dialogSchliessen = !result.equals(leereAuswahl) && !result.equals(leererString);
            }
        }
        
        return result;
    }
    
    public byte readByte (String title, String message)
    {
        byte result = 0;
        String eingabe = "";
        boolean dialogSchliessen = false;
        
        while (!dialogSchliessen)
        {
            eingabe = showInputDialogComboBox(title, message, null);
            try
            {
                if (selectedOption == CANCEL_OPTION)
                {
                    dialogSchliessen = true;
                }
                else
                {
                    result = Byte.parseByte(eingabe);
                    dialogSchliessen = true;
                }
            }
            catch (NumberFormatException e)
            {
                dialogSchliessen = false;
            }
        }
        
        return result;
    }
    
    public short readShort (String title, String message)
    {
        short result = 0;
        String eingabe = "";
        boolean dialogSchliessen = false;
        
        while (!dialogSchliessen)
        {
            eingabe = showInputDialogComboBox(title, message, null);
            try
            {
                if (selectedOption == CANCEL_OPTION)
                {
                    dialogSchliessen = true;
                }
                else
                {
                    result = Short.parseShort(eingabe);
                    dialogSchliessen = true;
                }
            }
            catch (NumberFormatException e)
            {
                dialogSchliessen = false;
            }
        }
        
        return result;
    }
    
    public int readInt (String title, String message)
    {
        int result = 0;
        String eingabe = "";
        boolean dialogSchliessen = false;
        
        while (!dialogSchliessen)
        {
            eingabe = showInputDialogComboBox(title, message, null);
            try
            {
                if (selectedOption == CANCEL_OPTION)
                {
                    dialogSchliessen = true;
                }
                else
                {
                    result = Integer.parseInt(eingabe);
                    dialogSchliessen = true;
                }
            }
            catch (NumberFormatException e)
            {
                dialogSchliessen = false;
            }
        }
        
        return result;
    }
    
    public long readLong (String title, String message)
    {
        long result = 0L;
        String eingabe = "";
        boolean dialogSchliessen = false;
        
        while (!dialogSchliessen)
        {
            eingabe = showInputDialogComboBox(title, message, null);
            try
            {
                if (selectedOption == CANCEL_OPTION)
                {
                    dialogSchliessen = true;
                }
                else
                {
                    result = Long.parseLong(eingabe);
                    dialogSchliessen = true;
                }
            }
            catch (NumberFormatException e)
            {
                dialogSchliessen = false;
            }
        }
        
        return result;
    }
    
    public float readFloat (String title, String message)
    {
        float result = 0;
        String eingabe = "";
        boolean dialogSchliessen = false;
        
        while (!dialogSchliessen)
        {
            eingabe = showInputDialogComboBox(title, message, null);
            try
            {
                if (selectedOption == CANCEL_OPTION)
                {
                    dialogSchliessen = true;
                }
                else
                {
                    result = Float.parseFloat(eingabe);
                    dialogSchliessen = true;
                }
            }
            catch (NumberFormatException e)
            {
                dialogSchliessen = false;
            }
        }
        
        return result;
    }
    
    public double readDouble (String title, String message)
    {
        double result = 0.0;
        String eingabe = "";
        boolean dialogSchliessen = false;
        
        while (!dialogSchliessen)
        {
            eingabe = showInputDialogComboBox(title, message, null);
            try
            {
                if (selectedOption == CANCEL_OPTION)
                {
                    dialogSchliessen = true;
                }
                else
                {
                    result = Double.parseDouble(eingabe);
                    dialogSchliessen = true;
                }
            }
            catch (NumberFormatException e)
            {
                dialogSchliessen = false;
            }
        }
        
        return result;
    }
    
    public char readChar (String title, String message)
    {
        char result = (char)0;
        String eingabe = "";
        boolean dialogSchliessen = false;
        
        while (!dialogSchliessen)
        {
            eingabe = showInputDialogComboBox(title, message, null);
            try
            {
                if (selectedOption == CANCEL_OPTION)
                {
                    dialogSchliessen = true;
                }
                else
                {
                    result = eingabe.charAt(0);
                    dialogSchliessen = true;
                }
            }
            catch (NumberFormatException e)
            {
                dialogSchliessen = false;
            }
        }
        
        return result;
    }
    
    public boolean readBoolean (String title, String message)
    {
        boolean result = false;
        String eingabe = "";
        boolean dialogSchliessen = false;
        String[] options = { "", "true", "false" };
        
        while (!dialogSchliessen)
        {
            eingabe = showInputDialogComboBox(title, message, options);
            try
            {
                if (selectedOption == CANCEL_OPTION)
                {
                    dialogSchliessen = true;
                }
                else
                {
                    result = Boolean.parseBoolean(eingabe);
                    dialogSchliessen = true;
                }
            }
            catch (NumberFormatException e)
            {
                dialogSchliessen = false;
            }
        }
        
        return result;
    }
    
    public String readLine (String title, String message)
    {
        String result = null;
        result = showInputDialogComboBox(title, message, null);
        return result;
    }
    
    public int showConfirmDialog (String title, String message)
    {
        int result = 0;
        result = JOptionPane.showConfirmDialog(this, message);
        selectedOption = result;
        return result;
    }
    
    public int showFileOpenDialog ()
    {
        int result = 0;
        
        fileChooser.setFileFilter(fileFilter);
        result = fileChooser.showOpenDialog(this);
        
        if (result == APPROVE_FILE)
        {
            choosenFile = fileChooser.getSelectedFile();
            selectedOption = APPROVE_FILE;
        }
        else
        {
            choosenFile = null;
            selectedOption = CANCEL_FILE;
        }
        
        return result;
    }
    
    public int showFileSaveDialog ()
    {
        int result = CLOSED_OPTION;
        
        fileChooser.setFileFilter(fileFilter);
        result = fileChooser.showSaveDialog(this);
        
        if (result == APPROVE_FILE)
        {
            choosenFile = fileChooser.getSelectedFile();
            selectedOption = APPROVE_FILE;
        }
        else
        {
            choosenFile = null;
            selectedOption = CANCEL_FILE;
        }
        
        return result;
    }
    
    public int showColorChooserDialog (String title)
    {
        int result = CLOSED_OPTION;
        
        choosenColor = JColorChooser.showDialog(this, title, null);
        
        if (choosenColor == null)
        {
            selectedOption = CANCEL_OPTION;
        }
        else
        {
            selectedOption = OK_OPTION;
        }
        result = selectedOption;
        return result;
    }
    
    public void resetMouseEvent ()
    {
        selectedMouseButton = ScreenSpec.MOUSEBUTTON_RESET_VALUE;
        selectedSource = null;
    }
    
    public void resetKeyEvent ()
    {
        selectedKeyCode = ScreenSpec.KEYCODE_RESET_VALUE;
        selectedKeyChar = ScreenSpec.KEYCHAR_RESET_VALUE;
    }
    
    
    /**
     * Innere Listener-Klasse fuer die Auswertung von Maus-Ereignissen.
     */
    class MouseListener extends MouseAdapter
    {
        /**
         * Kommando zum Auswerten eines Mausklicks.
         *
         * Als Seiteneffekt veraendern sich der Wert der Abfragen getSelectedMouseButton
         * und getSelectedCell.
         * 
         * @param event Mausklick-Ereignis, das ausgewertet werden soll
         */
        public void mousePressed (MouseEvent event)
        {
            int button = event.getButton();
            selectedSource = event.getSource();
            
            switch (button)
            {
                case MouseEvent.BUTTON1 :
                    selectedMouseButton = MouseEvent.BUTTON1;
                    if (debug)
                    {
                        System.out.println("Screen.mousePressed: BUTTON1 (linke Maustaste) = " + MouseEvent.BUTTON1);
                    }
                    break;
                case MouseEvent.BUTTON2 :
                    selectedMouseButton = MouseEvent.BUTTON2;
                    if (debug)
                    {
                        System.out.println("Screen.mousePressed: BUTTON2 (mittlere Maustaste) = " + MouseEvent.BUTTON2);
                    }
                    break;
                case MouseEvent.BUTTON3 :
                    selectedMouseButton = MouseEvent.BUTTON3;
                    if (debug)
                    {
                        System.out.println("Screen.mousePressed : BUTTON3 (rechte Maustaste) = " + MouseEvent.BUTTON3);
                    }
                    break;
                default : selectedMouseButton = ScreenSpec.MOUSEBUTTON_RESET_VALUE;
            }
            
            if (debug && selectedSource != null)
            {
                System.out.println("Screen.mousePressed : selectedSource = " + selectedSource);
            }
        }
    }
    
    /**
     * Innere Listener-Klasse fuer die Auswertung von Tastatur-Ereignissen.
     */
    class KeyListener extends KeyAdapter
    {
        /**
         * Kommando zum Auswerten von Tastatureingaben.
         *
         * Als Seiteneffekt veraendern sich der Wert der Abfragen getSelectedKeyChar und getSelectedKeyCode.
         * 
         * @param event Tastatur-Ereignis, das ausgewertet werden soll
         */
        public void keyPressed (KeyEvent event)
        {
            selectedKeyCode = event.getKeyCode();
            selectedKeyChar = event.getKeyChar();
            if (debug)
            {
                System.out.print("Screen.keyPressed: selectedKeyCode = " + selectedKeyCode);
                System.out.println(" : selectedKeyChar = " + selectedKeyChar);
            }
        }
    }
}
