package Prog1Tools;

import javax.swing.*;
import java.awt.*;
import net.sourceforge.c4j.*;

/** 
 * Bildschirmzelle zur visuell ansprechenden Darstellung eines Zeichens.
 * 
 * @version 1.0
 * @author  hagen.buchwald@kit.edu
 *
 */
@SuppressWarnings("serial")
@ContractReference (contractClassName = "ScreenCellContract")
public class ScreenCell extends JLabel
{
    private int row;
    private int column;
    private Color foregroundColor;
    private Color backgroundColor;
    
    /**
     * Konstruktor zum Erzeugen einer Bildschirmzelle. Bildschirmzelle, die in eine Umgebung (z.B. Farbbildschirm) eingefuegt werden kann
     * 
     * @param row Zeilenangabe der Position der Bildschirmzelle in Ihrer Umgebung
     * @param column Spaltenangabe der Position der Bildschirmzelle in Ihrer Umgebung
     * @param character darzustellendes Zeichen
     *
     */
    public ScreenCell (int row, int column, Character character)
    {
        super(character.toString());
        this.row = row;
        this.column = column;
        this.foregroundColor = new Color(Screen.BLACK_R, Screen.BLACK_G, Screen.BLACK_B);
        this.backgroundColor = new Color(Screen.GREY_R, Screen.GREY_G, Screen.GREY_B);
        this.setForeground(foregroundColor);
        this.setBackground(backgroundColor);
        this.setHorizontalTextPosition(JLabel.CENTER);
        this.setHorizontalAlignment(JLabel.CENTER);
        this.setOpaque(true);
    }
    
    /**
     * Abfrage der Zeilenangabe der Position der Bildschirmzelle in ihrer Umgebung.
     * 
     * @return Zeilenangabe der Position der Bildschirmzelle
     */
    @Pure
    public int getRow ()
    {
        return row;
    }
    
    /**
     * Abfrage der Spaltenangabe der Position der Bildschirmzelle in ihrer Umgebung.
     * 
     * @return Spaltenangabe der Position der Bildschirmzelle
     */
    @Pure
    public int getColumn ()
    {
        return column;
    }
    
    /**
     * Abfrage des Zeichens, das von der Bildschirmzelle angezeigt wird.
     * 
     * @return Zeichen, das von der Bildschirmzelle angezeigt wird
     */
    @Pure
    public char getChar ()
    {
        char result;
        result = this.getText().charAt(0);
        return result;
    }
    
    /**
     * Kommando zum Setzen der Vordergrundfarbe der Bildschirmzelle.
     * 
     * @param red Rot-Anteil des RGB-Wertes der Vordergrundfarbe
     * @param green Gruen-Anteil des RGB-Wertes der Vordergrundfarbe
     * @param blue Blau-Anteil des RGB-Wertes der Vordergrundfarbe
     */
    public void setForeground (int red, int green, int blue)
    {
        this.foregroundColor = new Color(red, green, blue);
        this.setForeground(foregroundColor);
    }
    
    /**
     * Kommando zum Setzen der Hintergrundfarbe der Bildschirmzelle.
     * 
     * @param red Rot-Anteil des RGB-Wertes der Hintergrundfarbe
     * @param green Gruen-Anteil des RGB-Wertes der Hintergrundfarbe
     * @param blue Blau-Anteil des RGB-Wertes der Hintergrundfarbe
     */
    public void setBackground (int red, int green, int blue)
    {
        this.backgroundColor = new Color(red, green, blue);
        this.setBackground(backgroundColor);
    }
    
    /**
     * Abfrage des Rot-Anteils des RGB-Wertes der Vordergrundfarbe der Bildschirmzelle.
     * 
     * @return Rot-Anteil des RGB-Wertes der Vordergrundfarbe
     */
    @Pure
    public int getForegroundRed ()
    {
        return foregroundColor.getRed();
    }
    
    /**
     * Abfrage des Gruen-Anteils des RGB-Wertes der Vordergrundfarbe der Bildschirmzelle.
     * 
     * @return Gruen-Anteil des RGB-Wertes der Vordergrundfarbe
     */
    @Pure
    public int getForegroundGreen ()
    {
        return foregroundColor.getGreen();
    }
    
    /**
     * Abfrage des Blau-Anteils des RGB-Wertes der Vordergrundfarbe der Bildschirmzelle.
     * 
     * @return Blau-Anteil des RGB-Wertes der Vordergrundfarbe
     */
    @Pure
    public int getForegroundBlue ()
    {
        return foregroundColor.getBlue();
    }
    
    /**
     * Abfrage des Rot-Anteils des RGB-Wertes der Hintergrundfarbe der Bildschirmzelle.
     * 
     * @return Rot-Anteil des RGB-Wertes der Hintergrundfarbe
     */
    @Pure
    public int getBackgroundRed ()
    {
        return backgroundColor.getRed();
    }
    
    /**
     * Abfrage des Gruen-Anteils des RGB-Wertes der Hintergrundfarbe der Bildschirmzelle.
     * 
     * @return Gruen-Anteil des RGB-Wertes der Hintergrundfarbe
     */
    @Pure
    public int getBackgroundGreen ()
    {
        return backgroundColor.getGreen();
    }
    
    /**
     * Abfrage des Blau-Anteils des RGB-Wertes der Hintergrundfarbe der Bildschirmzelle.
     * 
     * @return Blau-Anteil des RGB-Wertes der Hintergrundfarbe
     */
    @Pure
    public int getBackgroundBlue ()
    {
        return backgroundColor.getBlue();
    }
    
    /**
     * Abfrage der textuellen Beschreibung des sichtbaren Zustandes der Bildschirmzelle.
     *
     * Die Reihenfolge der Felder lautet: row, column, char, foregroundRed, foregroundGreen, foregroundBlue, backgroundRed, backgroudnGreen, backgroundBlue.
     * Als Trennzeichen dient das Standard-Trennzeichen, das auch fr CSV-Dateien verwendet wird.
     * 
     * @return textuelle Beschreibung des sichtbaren Zustandes der Bildschirmzelle 
     */
    @Pure
    public String toString ()
    {
        String result = "";
        
        result = result + getRow() + Screen.DELIMITER;
        result = result + getColumn() + Screen.DELIMITER;
        result = result + getChar() + Screen.DELIMITER;
        result = result + getForegroundRed() + Screen.DELIMITER;
        result = result + getForegroundGreen() + Screen.DELIMITER;
        result = result + getForegroundBlue() + Screen.DELIMITER;
        result = result + getBackgroundRed() + Screen.DELIMITER;
        result = result + getBackgroundGreen() + Screen.DELIMITER;
        result = result + getBackgroundBlue();
        
        return result;
    }
}
    
