package Prog1Tools;

import java.awt.*;
import javax.swing.*;
import java.io.*;
import net.sourceforge.c4j.*;

/** 
 * Das Interface ScreenSpec spezifiziert Standards wie die KIT-Farben
 * und grundlegende Funktionen zum Umgang mit einem Screen.
 * 
 * @author  hagen.buchwald@kit.edu
 * @version 2.0
 * @date 2010-09-03
 */
@ContractReference (contractClassName = "ScreenSpecContract")
public interface ScreenSpec
{
    
    /** Anzahl der Text-Zeilen der Standard-Aufloesung eines Text-Screens. */
    public static final int ROWS = 25;
    
    /** Anzahl der Text-Spalten der Standard-Aufloesung eines Text-Screens. */
    public static final int COLUMNS = 80;
    
    /** Breite eines GraphicScreens in der Standard-Aufloesung. */
    public static final int MAX_X = 800;
    
    /** Hoehe eines GraphicScreens in der Standard-Aufloesung. */
    public static final int MAX_Y = 600;
    
    /** Trennzeichen der CSV-Dateien, in denen Screen-Abbilder abgespeichert sind. */
    public static final char DELIMITER = ';';
    
    /** Rueckgabewert eines Dialogfensters, wenn es mit einem Ja geschlossen wurde.*/
    public static final int OK_OPTION = JOptionPane.OK_OPTION;;
    
    /** Rueckgabewert eines Dialogfensters, wenn es mit einem Ja geschlossen wurde.*/
    public static final int YES_OPTION = JOptionPane.YES_OPTION;;
    
    /** Rueckgabewert eines Dialogfensters, wenn es mit einem Nein geschlossen wurde.*/
    public static final int NO_OPTION = JOptionPane.NO_OPTION;;
    
    /** Rueckgabewert eines Dialogfensters, wenn es mit einem Abbruch geschlossen wurde.*/
    public static final int CANCEL_OPTION = JOptionPane.CANCEL_OPTION;;
    
    /** Rueckgabewert eines Dialogfensters, wenn es via Fenster-Schlieen-Schalter geschlossen wurde.*/
    public static final int CLOSED_OPTION = JOptionPane.CANCEL_OPTION;;
    
    /** Rueckgabewert eines Dialogfensters, wenn es mit einer Bestaetigung geschlossen wurde.*/
    public static final int APPROVE_FILE = JFileChooser.APPROVE_OPTION;
    
    /** Rueckgabewert eines Dialogfensters, wenn es mit einem Abbruch geschlossen wurde.*/
    public static final int CANCEL_FILE = JFileChooser.CANCEL_OPTION;;
    
    /** Wert, auf den selectedMouseButton durch das Kommando reset zurueckgesetzt wird. */
    public final int MOUSEBUTTON_RESET_VALUE = 0;
    
    /** Wert, auf den selectedKeyCode durch das Kommando reset zurueckgesetzt wird. */
    public final int KEYCODE_RESET_VALUE = 0;
    
    /** Wert, auf den selectedKeyChar durch das Kommando reset zurueckgesetzt wird. */
    public final char KEYCHAR_RESET_VALUE = ' ';
    
    /** RGB-Wert des Rot-Anteils der KIT-Farbe Scharz. */
    public static final int BLACK_R = 0;
    /** RGB-Wert des Gruen-Anteils der KIT-Farbe Scharz. */
    public static final int BLACK_G = 0;
    /** RGB-Wert des Blau-Anteils der KIT-Farbe Scharz. */
    public static final int BLACK_B = 0;
    
    /** RGB-Wert des Rot-Anteils der KIT-Farbe Weiss. */
    public static final int WHITE_R = 255;
    /** RGB-Wert des Gruen-Anteils der KIT-Farbe Weiss. */
    public static final int WHITE_G = 255;
    /** RGB-Wert des Blau-Anteils der KIT-Farbe Weiss. */
    public static final int WHITE_B = 255;
    
    /** RGB-Wert des Rot-Anteils der KIT-Farbe Grau. */
    public static final int GREY_R = 217;
    /** RGB-Wert des Gruen-Anteils der KIT-Farbe Grau. */
    public static final int GREY_G = 217;
    /** RGB-Wert des Blau-Anteils der KIT-Farbe Grau. */
    public static final int GREY_B = 217;
    
    /** RGB-Wert des Rot-Anteils der KIT-Farbe Dunkelgrau. */
    public static final int GREY2_R = 128;
    /** RGB-Wert des Gruen-Anteils der KIT-Farbe Dunkelgrau. */
    public static final int GREY2_G = 128;
    /** RGB-Wert des Blau-Anteils der KIT-Farbe Dunkelgrau. */
    public static final int GREY2_B = 128;
    
    /** RGB-Wert des Rot-Anteils der KIT-Farbe Gruen. */
    public static final int GREEN_R = 0;
    /** RGB-Wert des Gruen-Anteils der KIT-Farbe Gruen. */
    public static final int GREEN_G = 150;
    /** RGB-Wert des Blau-Anteils der KIT-Farbe Gruen. */
    public static final int GREEN_B = 130;
    
    /** RGB-Wert des Rot-Anteils der KIT-Farbe Hellgruen. */
    public static final int GREEN2_R = 130;
    /** RGB-Wert des Gruen-Anteils der KIT-Farbe Hellgruen. */
    public static final int GREEN2_G = 190;
    /** RGB-Wert des Blau-Anteils der KIT-Farbe Hellgruen. */
    public static final int GREEN2_B = 60;
    
    /** RGB-Wert des Rot-Anteils der KIT-Farbe Blau. */
    public static final int BLUE_R = 70;
    /** RGB-Wert des Gruen-Anteils der KIT-Farbe Blau. */
    public static final int BLUE_G = 100;
    /** RGB-Wert des Blau-Anteils der KIT-Farbe Blau. */
    public static final int BLUE_B = 170;
    
    /** RGB-Wert des Rot-Anteils der KIT-Farbe Dunkelblau. */
    public static final int BLUE2_R = 125;
    /** RGB-Wert des Gruen-Anteils der KIT-Farbe Dunkelblau. */
    public static final int BLUE2_G = 146;
    /** RGB-Wert des Blau-Anteils der KIT-Farbe Dunkelblau. */
    public static final int BLUE2_B = 195;
    
    /** RGB-Wert des Rot-Anteils der KIT-Farbe Hellblau. */
    public static final int BLUE3_R = 80;
    /** RGB-Wert des Gruen-Anteils der KIT-Farbe Hellblau. */
    public static final int BLUE3_G = 170;
    /** RGB-Wert des Blau-Anteils der KIT-Farbe Hellblau. */
    public static final int BLUE3_B = 230;
    
    /** RGB-Wert des Rot-Anteils der KIT-Farbe Orange. */
    public static final int ORANGE_R = 250;
    /** RGB-Wert des Gruen-Anteils der KIT-Farbe Orange. */
    public static final int ORANGE_G = 130;
    /** RGB-Wert des Blau-Anteils der KIT-Farbe Orange. */
    public static final int ORANGE_B = 20;
    
    /** RGB-Wert des Rot-Anteils der KIT-Farbe Hellbraun. */
    public static final int BROWN_R = 220;
    /** RGB-Wert des Gruen-Anteils der KIT-Farbe Hellbraun. */
    public static final int BROWN_G = 160;
    /** RGB-Wert des Blau-Anteils der KIT-Farbe Hellbraun. */
    public static final int BROWN_B = 30;
    
    /** RGB-Wert des Rot-Anteils der KIT-Farbe Dunkelbraun. */
    public static final int BROWN2_R = 160;
    /** RGB-Wert des Gruen-Anteils der KIT-Farbe Dunkelbraun. */
    public static final int BROWN2_G = 130;
    /** RGB-Wert des Blau-Anteils der KIT-Farbe Dunkelbraun. */
    public static final int BROWN2_B = 50;
    
    /** RGB-Wert des Rot-Anteils der KIT-Farbe Rot. */
    public static final int RED_R = 160;
    /** RGB-Wert des Gruen-Anteils der KIT-Farbe Rot. */
    public static final int RED_G = 30;
    /** RGB-Wert des Blau-Anteils der KIT-Farbe Rot. */
    public static final int RED_B = 40;
    
    /** RGB-Wert des Rot-Anteils der KIT-Farbe Lila. */
    public static final int PURPLE_R = 160;
    /** RGB-Wert des Gruen-Anteils der KIT-Farbe Lila. */
    public static final int PURPLE_G = 0;
    /** RGB-Wert des Blau-Anteils der KIT-Farbe Lila. */
    public static final int PURPLE_B = 120;
    
    /** KIT-Scharz */
    public static final Color BLACK = new Color(BLACK_R, BLACK_G, BLACK_B);
    
    /** KIT-Weiss */
    public static final Color WHITE = new Color(WHITE_R, WHITE_G, WHITE_B);
    
    /** KIT-Grau */
    public static final Color GREY = new Color(GREY_R, GREY_G, GREY_B);
    
    /** KIT-Dunkelgrau */
    public static final Color GREY2 = new Color(GREY2_R, GREY2_G, GREY2_B);
    
    /** KIT-Gruen */
    public static final Color GREEN = new Color(GREEN_R, GREEN_G, GREEN_B);
    
    /** KIT-Hellgruen */
    public static final Color GREEN2 = new Color(GREEN2_R, GREEN2_G, GREEN2_B);
    
    /** KIT-Blau */
    public static final Color BLUE = new Color(BLUE_R, BLUE_G, BLUE_B);
    
    /** KIT-Dunkelblau */
    public static final Color BLUE2 = new Color(BLUE2_R, BLUE2_G, BLUE2_B);
    
    /** KIT-Hellblau */
    public static final Color BLUE3 = new Color(BLUE3_R, BLUE3_G, BLUE3_B);
    
    /** KIT-Orange */
    public static final Color ORANGE = new Color(ORANGE_R, ORANGE_G, ORANGE_B);
    
    /** KIT-Hellbraun */
    public static final Color BROWN = new Color(BROWN_R, BROWN_G, BROWN_B);
    
    /** KIT-Dunkelbraun */
    public static final Color BROWN2 = new Color(BROWN2_R, BROWN2_G, BROWN2_B);
    
    /** KIT-Rot */
    public static final Color RED = new Color(RED_R, RED_G, RED_B);
    
    /** KIT-Lila */
    public static final Color PURPLE = new Color(PURPLE_R, PURPLE_G, PURPLE_B);
    
    /** Standard-Schriftart */
    public static final String FONT_NAME = "DialogInput";
    
    /** Standard-Schriftstil */
    public static final int FONT_STYLE = Font.PLAIN;
    
    /** Standard-Schriftgroesse */
    public static final int FONT_SIZE = 14;
    
    /**
     * Abfrage der zuletzt gedrueckten Maus-Taste.
     * 
     * @return MouseEvent.BUTTON1 fuer die linke Maustaste,
     *         MouseEvent.BUTTON2 fuer die mittlere Maustaste,
     *         MouseEvent.BUTTON3 fuer die rechte Maustaste
     */
    public int getSelectedMouseButton ();
    
    /**
     * Abfrage des KeyCodes der zuletzt gedrueckten Tastatur-Taste.
     * 
     * @return KeyCode der Tastatur-Taste, z.B. KeyEvent.VK_A, wenn die A-Taste gedrueckt wurde.
     */
    public int getSelectedKeyCode ();
    
    /**
     * Abfrage des Zeichens der zuletzt gedrueckten Tastatur-Taste.
     * 
     * @return KeyChar der Tastatur-Taste, z.B. 'A', wenn die A-Taste gedrueckt wurde.
     */
    public char getSelectedKeyChar ();
    
    /**
     * Abfrage, ob das zuletzt geschlossene Dialog-Fenster
     * mit einer Bestaetigung oder einem Abbruch geschlossen wurde.
     */
    public int getSelectedOption ();
    
    /**
     * Abfrage der Farbe, die in dem zuletzt durch Bestaetigung geschlossenen Farbauswahl-Dialog-Fenster ausgewaehlt wurde.
     * 
     * @return Farbe, die im Farbauswahl-Dialog-Fenster ausgewaehlten wurde;
     *         null, wenn keine Farbe ausgewaehlt wurde
     */
    public Color getChoosenColor ();
    
    /**
     * Abfrage der Datei, die in dem zuletzt durch Bestaetigung geschlossenen Datei-Dialog-Fenster ausgewaehlt wurde.
     * 
     * @return Datei, die im Datei-Dialog-Fenster ausgewaehlten wurde;
     *         null, wenn keine Datei ausgewaehlt wurde
     */
    public File getChoosenFile ();
    
    /**
     * Kommando zum Setzen des Debug-Modus.
     * 
     * @param debug falls true, werden Debug-Informationen auf der Konsole ausgegeben; falls false, werden keine Debug-Informationen auf der Konsole ausgegeben
     */
    public void setDebug (boolean debug);
    
    /**
     * Kommando zum Festlegen, ob der Screen in seiner Groesse veraendert werden darf.
     * 
     * @param resizable fuer true ist der Farbbildschirm in seiner Groesse veraenderbar
     */
    public void setResizable (boolean resizable);
    
    /**
     * Kommando zum Zuruecksetzen der zur Anzeige der Zeichen verwendeten Schriftart.
     */
    public void resetFont ();
    
    /**
     * Kommando zum Setzen der zur Anzeige der Zeichen verwendeten Schriftart.
     * 
     * @param fontName Schriftart
     * @param fontStyle Schriftstil
     * @param fontSize Schriftgroesse
     */
    public void setFont (String fontName, int fontStyle, int fontSize);
    
    /**
     * Kommando zum Loeschen der Anzeige.
     * 
     */
    public void clearScreen ();
    
    /**
     * Kommando zum Loeschen der Anzeige.
     * 
     * @param fastMode falls true, wird der Inhalt des Screens kurz unsichtbar, um das Loeschen zu beschleunigen
     */
    public void clearScreen (boolean fastMode);
    
    /**
     * Kommando, ein Dialogfenster vom Typ MessageDialog anzuzeigen.
     * Als Seiteneffekt wird die Variable selectedOption auf den Wert APPROVE_OPTION gesetzt.
     * 
     * @param title Text, der im Titelbereich des Dialogfensters angezeigt werden soll
     * @param message Text, der im Dialogfenster angezeigt werden soll.
     */
    public void showMessageDialog (String title, String message);
    
    
    /**
     * Kommando, ein Dialogfenster vom Typ OptionDialog mit den Optionen Abbruch und OK anzuzeigen.
     * 
     * @param title Text, der im Titelbereich des Dialogfensters angezeigt werden soll
     * @param message Text, der im Dialogfenster angezeigt werden soll.
     *
     * @return APPROVE_OPTION, wenn das Fenster mit einer Bestaetigung (OK) geschlossen wurde, CANCEL_OPTION sonst
     */
    public int showOptionDialogAbbruchOK (String title, String message);
    
    
    /**
     * Kommando, ein Dialogfenster vom Typ OptionDialog mit den Optionen Ja und Nein anzuzeigen.
     * 
     * @param title Text, der im Titelbereich des Dialogfensters angezeigt werden soll
     * @param message Text, der im Dialogfenster angezeigt werden soll
     *
     * @return APPROVE_OPTION, wenn das Fenster mit einer Bestaetigung (Ja) geschlossen wurde, CANCEL_OPTION sonst
     */
    public int showOptionDialogJaNein (String title, String message);
    
    /**
     * Kommando, ein Dialogfenster vom Typ InputDialog mit einer ComboBox anzuzeigen.
     * 
     * @param title Text, der im Titelbereich des Dialogfensters angezeigt werden soll
     * @param message Text, der im Dialogfenster angezeigt werden soll
     * @param comboBoxItems ein-dimensionales Feld, das die in der ComboBox anzuzeigenden Optionen enthaelt
     *
     * @return Option, die in der ComboBox selektiert wurde und durch den Bestaetigen-Schalter ausgewaehlt wurde
     */
    public String showInputDialogComboBox (String title, String message, String[] comboBoxItems);
    
    /**
     * Abfrage zum Einlesen eines byte Wertes via Dialogfenster.
     * Das Dialogfenster kann erst geschlossen werden, wenn ein zulaessiger Wert eingegeben wurde
     * oder der Abbruch-Schalter gedrueckt wurde.
     *
     * Als Seiteneffekt veraendert sich der Wert der Abfrage getSelectedOption.
     * Wurde das Dialogfenster durch den Bestaetigen-Schalter geschlossen, liefert getSelectedOption den Wert APPROVE_OPTION.
     * Wurde das Dialogfenster durch den Abbruch-Schalter geschlossen, liefert getSelectedOption den Wert CANCEL_OPTION.
     * 
     * @param title Text, der im Titelbereich des Dialogfensters angezeigt werden soll
     * @param message Text, der im Dialogfenster angezeigt werden soll
     *
     * @return Wert, der im Textfeld eingegeben wurde
     */
    public byte readByte (String title, String message);
    
    /**
     * Abfrage zum Einlesen eines short Wertes via Dialogfenster.
     * Das Dialogfenster kann erst geschlossen werden, wenn ein zulaessiger Wert eingegeben wurde
     * oder der Abbruch-Schalter gedrueckt wurde.
     *
     * Als Seiteneffekt veraendert sich der Wert der Abfrage getSelectedOption.
     * Wurde das Dialogfenster durch den Bestaetigen-Schalter geschlossen, liefert getSelectedOption den Wert APPROVE_OPTION.
     * Wurde das Dialogfenster durch den Abbruch-Schalter geschlossen, liefert getSelectedOption den Wert CANCEL_OPTION.
     * 
     * @param title Text, der im Titelbereich des Dialogfensters angezeigt werden soll
     * @param message Text, der im Dialogfenster angezeigt werden soll
     *
     * @return Wert, der im Textfeld eingegeben wurde
     */
    public short readShort (String title, String message);
    
    /**
     * Abfrage zum Einlesen eines int Wertes via Dialogfenster.
     * Das Dialogfenster kann erst geschlossen werden, wenn ein zulaessiger Wert eingegeben wurde
     * oder der Abbruch-Schalter gedrueckt wurde.
     *
     * Als Seiteneffekt veraendert sich der Wert der Abfrage getSelectedOption.
     * Wurde das Dialogfenster durch den Bestaetigen-Schalter geschlossen, liefert getSelectedOption den Wert APPROVE_OPTION.
     * Wurde das Dialogfenster durch den Abbruch-Schalter geschlossen, liefert getSelectedOption den Wert CANCEL_OPTION.
     * 
     * @param title Text, der im Titelbereich des Dialogfensters angezeigt werden soll
     * @param message Text, der im Dialogfenster angezeigt werden soll
     *
     * @return Wert, der im Textfeld eingegeben wurde
     */
    public int readInt (String title, String message);
    
    /**
     * Abfrage zum Einlesen eines long Wertes via Dialogfenster.
     * Das Dialogfenster kann erst geschlossen werden, wenn ein zulaessiger Wert eingegeben wurde
     * oder der Abbruch-Schalter gedrueckt wurde.
     *
     * Als Seiteneffekt veraendert sich der Wert der Abfrage getSelectedOption.
     * Wurde das Dialogfenster durch den Bestaetigen-Schalter geschlossen, liefert getSelectedOption den Wert APPROVE_OPTION.
     * Wurde das Dialogfenster durch den Abbruch-Schalter geschlossen, liefert getSelectedOption den Wert CANCEL_OPTION.
     * 
     * @param title Text, der im Titelbereich des Dialogfensters angezeigt werden soll
     * @param message Text, der im Dialogfenster angezeigt werden soll
     *
     * @return Wert, der im Textfeld eingegeben wurde
     */
    public long readLong (String title, String message);
    
    /**
     * Abfrage zum Einlesen eines float Wertes via Dialogfenster.
     * Das Dialogfenster kann erst geschlossen werden, wenn ein zulaessiger Wert eingegeben wurde
     * oder der Abbruch-Schalter gedrueckt wurde.
     *
     * Als Seiteneffekt veraendert sich der Wert der Abfrage getSelectedOption.
     * Wurde das Dialogfenster durch den Bestaetigen-Schalter geschlossen, liefert getSelectedOption den Wert APPROVE_OPTION.
     * Wurde das Dialogfenster durch den Abbruch-Schalter geschlossen, liefert getSelectedOption den Wert CANCEL_OPTION.
     * 
     * @param title Text, der im Titelbereich des Dialogfensters angezeigt werden soll
     * @param message Text, der im Dialogfenster angezeigt werden soll
     *
     * @return Wert, der im Textfeld eingegeben wurde
     */
    public float readFloat (String title, String message);
    
    /**
     * Abfrage zum Einlesen eines double Wertes via Dialogfenster.
     * Das Dialogfenster kann erst geschlossen werden, wenn ein zulaessiger Wert eingegeben wurde
     * oder der Abbruch-Schalter gedrueckt wurde.
     *
     * Als Seiteneffekt veraendert sich der Wert der Abfrage getSelectedOption.
     * Wurde das Dialogfenster durch den Bestaetigen-Schalter geschlossen, liefert getSelectedOption den Wert APPROVE_OPTION.
     * Wurde das Dialogfenster durch den Abbruch-Schalter geschlossen, liefert getSelectedOption den Wert CANCEL_OPTION.
     * 
     * @param title Text, der im Titelbereich des Dialogfensters angezeigt werden soll
     * @param message Text, der im Dialogfenster angezeigt werden soll
     *
     * @return Wert, der im Textfeld eingegeben wurde
     */
    public double readDouble (String title, String message);
    
    /**
     * Abfrage zum Einlesen eines char Wertes via Dialogfenster.
     * Wird mehr als ein Zeichen eingegeben, wird nur das erste Zeichen verwendet, der Rest
     * der Eingabe wird abgeschnitten.
     *
     * Als Seiteneffekt veraendert sich der Wert der Abfrage getSelectedOption.
     * Wurde das Dialogfenster durch den Bestaetigen-Schalter geschlossen, liefert getSelectedOption den Wert APPROVE_OPTION.
     * Wurde das Dialogfenster durch den Abbruch-Schalter geschlossen, liefert getSelectedOption den Wert CANCEL_OPTION.
     * 
     * @param title Text, der im Titelbereich des Dialogfensters angezeigt werden soll
     * @param message Text, der im Dialogfenster angezeigt werden soll
     *
     * @return Zeichen, das im Textfeld eingegeben wurde
     */
    public char readChar (String title, String message);
    
    /**
     * Abfrage zum Einlesen eines boolean Wertes via Dialogfenster.
     * Das Dialogfenster kann erst geschlossen werden, wenn eine Option in der
     * ComboBox ausgewaehlt wurde oder der Abbruch-Schalter gedrueckt wurde.
     *
     * Als Seiteneffekt veraendert sich der Wert der Abfrage getSelectedOption.
     * Wurde das Dialogfenster durch den Bestaetigen-Schalter geschlossen, liefert getSelectedOption den Wert APPROVE_OPTION.
     * Wurde das Dialogfenster durch den Abbruch-Schalter geschlossen, liefert getSelectedOption den Wert CANCEL_OPTION.
     * 
     * @param title Text, der im Titelbereich des Dialogfensters angezeigt werden soll
     * @param message Text, der im Dialogfenster angezeigt werden soll
     *
     * @return boolescher Wert, der in der ComBoBox ausgewaehlt wurde
     */
    public boolean readBoolean (String title, String message);
    
    /**
     * Abfrage zum Einlesen einer ganzen Text-Zeile via Dialogfenster.
     * Das Dialogfenster kann erst geschlossen werden, wenn ein zulaessiger Wert eingegeben wurde
     * oder der Abbruch-Schalter gedrueckt wurde.
     *
     * Als Seiteneffekt veraendert sich der Wert der Abfrage getSelectedOption.
     * Wurde das Dialogfenster durch den Bestaetigen-Schalter geschlossen, liefert getSelectedOption den Wert APPROVE_OPTION.
     * Wurde das Dialogfenster durch den Abbruch-Schalter geschlossen, liefert getSelectedOption den Wert CANCEL_OPTION.
     * 
     * @param title Text, der im Titelbereich des Dialogfensters angezeigt werden soll
     * @param message Text, der im Dialogfenster angezeigt werden soll
     *
     * @return Zeichen, das im Textfeld eingegeben wurde
     */
    public String readLine (String title, String message);
    
    /**
     * Kommando, ein Dialogfenster vom Typ ConfirmDialog anzuzeigen,
     * um einen Vorgang abschliessend zu bestaetigen oder doch noch abzubrechen.
     *
     * Als Seiteneffekt veraendert sich der Wert der Abfrage getSelectedOption.
     * Wurde das Dialogfenster durch den Bestaetigen-Schalter geschlossen, liefert getSelectedOption den Wert APPROVE_OPTION.
     * Wurde das Dialogfenster durch den Abbruch-Schalter geschlossen, liefert getSelectedOption den Wert CANCEL_OPTION.
     * 
     * @param title Text, der im Titelbereich des Dialogfensters angezeigt werden soll
     * @param message Text, der im Dialogfenster angezeigt werden soll
     *
     * @return APPROVE_OPTION, wenn der Bestaetigen-Schalter gedrueckt wurde, CANCEL_OPTION sonst
     */
    public int showConfirmDialog (String title, String message);
    
    /**
     * Kommando, ein Dialogfenster vom Typ FileOpenDialog anzuzeigen, um eine Datei auszuwaehlen,
     * die in einem naechsten Schritt geoeffnet werden soll.
     *
     * Als Seiteneffekt veraendert sich der Wert der Abfrage getSelectedOption.
     * Wurde das Dialogfenster durch den Bestaetigen-Schalter geschlossen, liefert getSelectedOption den Wert APPROVE_OPTION.
     * Wurde das Dialogfenster durch den Abbruch-Schalter geschlossen, liefert getSelectedOption den Wert CANCEL_OPTION.
     * 
     * Als weiterer Seiteneffekt veraendert sich der Wert der Abfrage getChoosenFile.
     * Wurde das Dialogfenster durch den Bestaetigen-Schalter geschlossen, liefert getChoosenFile die ausgewaehlte Datei.
     * Wurde das Dialogfenster durch den Abbruch-Schalter geschlossen, liefert getChoosenFile den Wert null.
     * 
     * @return APPROVE_OPTION, wenn der Bestaetigen-Schalter gedrueckt wurde, CANCEL_OPTION sonst
     */
    public int showFileOpenDialog ();
    
    /**
     * Kommando, ein Dialogfenster vom Typ FileOpenDialog anzuzeigen, um eine Datei auszuwaehlen,
     * die in einem naechsten Schritt abgespeichert werden soll.
     *
     * Als Seiteneffekt veraendert sich der Wert der Abfrage getSelectedOption.
     * Wurde das Dialogfenster durch den Bestaetigen-Schalter geschlossen, liefert getSelectedOption den Wert APPROVE_OPTION.
     * Wurde das Dialogfenster durch den Abbruch-Schalter geschlossen, liefert getSelectedOption den Wert CANCEL_OPTION.
     * 
     * Als weiterer Seiteneffekt veraendert sich der Wert der Abfrage getChoosenFile.
     * Wurde das Dialogfenster durch den Bestaetigen-Schalter geschlossen, liefert getChoosenFile die ausgewaehlte Datei.
     * Wurde das Dialogfenster durch den Abbruch-Schalter geschlossen, liefert getChoosenFile den Wert null.
     * 
     * @return APPROVE_OPTION, wenn der Bestaetigen-Schalter gedrueckt wurde, CANCEL_OPTION sonst
     */
    public int showFileSaveDialog ();
    
    /**
     * Kommando, ein Dialogfenster vom Typ ColorChooserDialog anzuzeigen,
     * um eine Farbe aus einer Farbpalette auszuwaehlen.
     *
     * Als Seiteneffekt veraendert sich der Wert der Abfrage getSelectedOption.
     * Wurde das Dialogfenster durch den Bestaetigen-Schalter geschlossen, liefert getSelectedOption den Wert APPROVE_OPTION.
     * Wurde das Dialogfenster durch den Abbruch-Schalter geschlossen, liefert getSelectedOption den Wert CANCEL_OPTION.
     * 
     * Als weiterer Seiteneffekt veraendert sich der Wert der Abfragen getChoosenColor.
     * Wurde das Dialogfenster durch den Bestaetigen-Schalter geschlossen, liefert getChoosenColor die ausgewaehlte Farbe.
     * Wurde das Dialogfenster durch den Abbruch-Schalter geschlossen, liefert getChoosenColor null.
     * 
     * @param title Text, der im Titelbereich des Dialogfensters angezeigt werden soll
     *
     * @return APPROVE_OPTION, wenn der Bestaetigen-Schalter gedrueckt wurde, CANCEL_OPTION sonst
     */
    public int showColorChooserDialog (String title);
    
    /**
     * Kommando, das letzte Mausklick-Ereignis zu loeschen.
     * Dieses Kommando sollte direkt nach der Auswertung des Mausklick-Ereignisses
     * ausgefuehrt werden, um das naechste Mausklick-Ereignis zulassen zu koennen.
     */
    public void resetMouseEvent ();
    
    /**
     * Kommando, das letzte Tastatur-Ereignis zu loeschen.
     * Dieses Kommando sollte direkt nach der Auswertung des Tastatur-Ereignisses
     * ausgefuehrt werden, um das naechste Tastatur-Ereignis zulassen zu koennen.
     */
    public void resetKeyEvent ();
}
