package Prog1Tools;

import java.awt.*;
import javax.swing.*;
import Prog1Tools.Screen;

/** 
 * Bildschirmzelle eines TextScreens.
 * 
 * @author  hagen.buchwald@kit.edu
 * @version 1.10
 * @date 2010-08-18
 */
@SuppressWarnings("serial")
public class TextScreenCell extends JLabel
                            implements TextScreenCellSpec
{
    /** Trennzeichen, das zur Trennung der Werte in der toString-Methode verwendet wird. */
    public static final char DELIMITER = ';';
    
    private int row;
    private int column;
    private Color foregroundColor;
    private Color backgroundColor;
    
    /**
     * Konstruktor zum Erzeugen einer Bildschirmzelle.
     * 
     * @param row Zeilenangabe der Position der Bildschirmzelle in Ihrer Umgebung
     * @param column Spaltenangabe der Position der Bildschirmzelle in Ihrer Umgebung
     * @param character darzustellendes Zeichen
     * @return Bildschirmzelle, die in eine Umgebung (z.B. Farbbildschirm) eingefuegt werden kann
     */
    public TextScreenCell (int row, int column, Character character)
    {
        super(character.toString());
        this.row = row;
        this.column = column;
        foregroundColor = Screen.BLACK;
        backgroundColor = Screen.GREY;
        Runnable initializeScreenCell = new Runnable()
        {
            public void run()
            {
                setForeground(foregroundColor);
                setBackground(backgroundColor);
                setHorizontalTextPosition(JLabel.CENTER);
                setHorizontalAlignment(JLabel.CENTER);
                setOpaque(true);
            }
        };
        try
        {
            SwingUtilities.invokeAndWait(initializeScreenCell);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
    
    /**
     * Abfrage der Zeilenangabe der Position der Bildschirmzelle in ihrer Umgebung.
     * 
     * @return Zeilenangabe der Position der Bildschirmzelle
     */
    public int getRow ()
    {
        int result = 0;
        result = row;
        return result;
    }
    
    /**
     * Abfrage der Spaltenangabe der Position der Bildschirmzelle in ihrer Umgebung.
     * 
     * @return Spaltenangabe der Position der Bildschirmzelle
     */
    public int getColumn ()
    {
        int result = 0;
        result = column;
        return result;
    }
    
    /**
     * Abfrage des Zeichens, das von der Bildschirmzelle angezeigt wird.
     * 
     * @return Zeichen, das von der Bildschirmzelle angezeigt wird
     */
    public char getChar ()
    {
        char result = ' ';
        result = this.getText().charAt(0);
        return result;
    }
    
    /**
     * Kommando zum Setzen des Zeichens, das in der Bildschirmzelle angezeigt werden soll.
     * 
     * @param character Zeichen, das von der Bildschirmzelle angezeigt werden soll
     */
    public void setChar (char character)
    {
        final Character text = character;
        Runnable setText = new Runnable()
        {
            public void run()
            {
                setText(text.toString());
            }
        };
        try
        {
            SwingUtilities.invokeAndWait(setText);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
    
    /**
     * Kommando zum Setzen der Vordergrundfarbe der Bildschirmzelle.
     * 
     * @param color Vordergrundfarbe der Bildschirmzelle
     */
    public void setForegroundColor (Color color)
    {
        this.foregroundColor = color;
        Runnable setForegroundColor = new Runnable()
        {
            public void run()
            {
                setForeground(foregroundColor);
            }
        };
        try
        {
            SwingUtilities.invokeAndWait(setForegroundColor);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
    
    /**
     * Kommando zum Setzen der Hintergrundfarbe der Bildschirmzelle.
     * 
     * @param color Hintergrundfarbe der Bildschirmzelle
     */
    public void setBackgroundColor (Color color)
    {
        this.backgroundColor = color;
        Runnable setBackgroundColor = new Runnable()
        {
            public void run()
            {
                setBackground(backgroundColor);
            }
        };
        try
        {
            SwingUtilities.invokeAndWait(setBackgroundColor);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
    
    /**
     * Abfrage der Vordergrundfarbe der Bildschirmzelle.
     * 
     * @return Vordergrundfarbe der Bildschirmzelle
     */
    public Color getForegroundColor ()
    {
        Color result;
        result = foregroundColor;
        return result;
    }
    
    /**
     * Abfrage der Hintergrundfarbe der Bildschirmzelle.
     * 
     * @return Hintergrundfarbe der Bildschirmzelle
     */
    public Color getBackgroundColor ()
    {
        Color result;
        result = backgroundColor;
        return result;
    }
    
    /**
     * Abfrage der textuellen Beschreibung des sichtbaren Zustandes der Bildschirmzelle.
     *
     * Die Reihenfolge der Felder lautet: row, column, char, foregroundRed, foregroundGreen, foregroundBlue, backgroundRed, backgroudnGreen, backgroundBlue.
     * Als Trennzeichen dient das Standard-Trennzeichen, das auch fr CSV-Dateien verwendet wird.
     * 
     * @return textuelle Beschreibung des sichtbaren Zustandes der Bildschirmzelle 
     */
    public String toString ()
    {
        String result = "";
        
        result = result + getRow() + DELIMITER;
        result = result + getColumn() + DELIMITER;
        result = result + getChar() + DELIMITER;
        result = result + getForegroundColor().getRed() + DELIMITER;
        result = result + getForegroundColor().getGreen() + DELIMITER;
        result = result + getForegroundColor().getBlue() + DELIMITER;
        result = result + getBackgroundColor().getRed() + DELIMITER;
        result = result + getBackgroundColor().getGreen() + DELIMITER;
        result = result + getBackgroundColor().getBlue();
        
        return result;
    }
}
    
