package Prog1Tools;

import java.awt.*;
import net.sourceforge.c4j.*;

/** 
 * Spezifikation einer Bildschirmzelle eines TextScreens
 * zur visuell ansprechenden Darstellung eines Zeichens.
 * 
 * @author  hagen.buchwald@kit.edu
 * @version 1.10
 * @date 2010-08-18
 */
@ContractReference (contractClassName = "TextScreenCellSpecContract")
public interface TextScreenCellSpec
{
    /**
     * Abfrage der Zeilenangabe der Position der Bildschirmzelle in ihrer Umgebung.
     * 
     * @return Zeilenangabe der Position der Bildschirmzelle
     */
    @Pure
    public int getRow ();
    
    /**
     * Abfrage der Spaltenangabe der Position der Bildschirmzelle in ihrer Umgebung.
     * 
     * @return Spaltenangabe der Position der Bildschirmzelle
     */
    @Pure
    public int getColumn ();
    
    /**
     * Abfrage des Zeichens, das von der Bildschirmzelle angezeigt wird.
     * 
     * @return Zeichen, das von der Bildschirmzelle angezeigt wird
     */
    @Pure
    public char getChar ();
    
    /**
     * Kommando zum Setzen des Zeichens, das in der Bildschirmzelle angezeigt werden soll.
     * 
     * @param character Zeichen, das von der Bildschirmzelle angezeigt werden soll
     */
    public void setChar (char character);
    
    /**
     * Kommando zum Setzen der Vordergrundfarbe der Bildschirmzelle.
     * 
     * @param color Vordergrundfarbe der Bildschirmzelle
     */
    public void setForegroundColor (Color color);
    
    /**
     * Kommando zum Setzen der Hintergrundfarbe der Bildschirmzelle.
     * 
     * @param color Hintergrundfarbe der Bildschirmzelle
     */
    public void setBackgroundColor (Color color);
    
    /**
     * Abfrage der Vordergrundfarbe der Bildschirmzelle.
     * 
     * @return Vordergrundfarbe der Bildschirmzelle
     */
    @Pure
    public Color getForegroundColor ();
    
    /**
     * Abfrage der Hintergrundfarbe der Bildschirmzelle.
     * 
     * @return Hintergrundfarbe der Bildschirmzelle
     */
    @Pure
    public Color getBackgroundColor ();
}
    
