/*
    Copyright (C) 2010  (Hagen Buchwald)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package Prog1Tools;

import Prog1Tools.GraphicScreen;
/** 
 * Die Klasse TurtleScreen erweitert die Klasse GraphicScreen,
 * so dass auf einfache Art und Weise mit Hilfe einer zeichnenden
 * Schildkrte grafische Darstellungen erstellt werden koennen.
 *
 * @author  hagen.buchwald@kit.edu
 * @version 1.0
 * @date 2010-11-14
 */
@SuppressWarnings("serial")
public class TurtleScreen extends GraphicScreen
                          implements TurtleSpec
{
    private static TurtleScreen turtleScreen;
    
    private int x1 = 0;
    private int y1 = 0;
    private int x2 = 0;
    private int y2 = 0;
    private int heading = 0;
    
    protected TurtleScreen (int width, int height)
    {
        super(width, height);
        setTitle("EJE TurtleScreen");
    }
    
    /**
     * Statische Methode zur individuellen Erzeugung eines graphischen Farbbildschirms
     * mit Turtle-Funktionalitaet.
     * Wiederholte Aufrufe dieser Methode liefern stets die gleiche Intanz (Singleton-Pattern).
     * 
     * @return singulaere Instanz der Klasse TurtleScreen
     */
    public synchronized static TurtleScreen getInstance (int width, int height)
    {
        TurtleScreen result = null;
        if (turtleScreen == null)
        {
            turtleScreen = new TurtleScreen(width, height);
        }
        result = turtleScreen;
        return result;
    }
    
    /**
     * Statische Methode zur individuellen Erzeugung eines graphischen Farbbildschirms
     * mit Turtle-Funktionalitaet.
     * Wiederholte Aufrufe dieser Methode liefern stets die gleiche Intanz (Singleton-Pattern).
     * 
     * @return singulaere Instanz der Klasse TurtleScreen
     */
    public synchronized static TurtleScreen getInstance ()
    {
        TurtleScreen result = null;
        result = getInstance(WIDTH, HEIGHT);
        return result;
    }
    
    public void setTurtle (int x, int y)
    {
        this.x1 = x;
        this.y1 = y;
    }
    
    public void setHeading (int heading)
    {
        if (heading < 0)
        {
            this.heading = 360 - (Math.abs(heading) % 360);
        }
        else
        {
            this.heading = heading % 360;
        }
        if (this.heading == 360)
        {
            this.heading = 0;
        }
    }
    
    public void forward (int distance)
    {
        x2 = x1 + (int)Math.round(distance * Math.cos(heading / 180.0 * Math.PI));
        y2 = y1 + (int)Math.round(distance * Math.sin(heading / 180.0 * Math.PI));
        drawLine(x1, y1, x2, y2);
        x1 = x2;
        y1 = y2;
    }
    
    public int getPositionX ()
    {
        int result = 0;
        result = x1;
        return result;
    }
    
    public int getPositionY ()
    {
        int result = 0;
        result = y1;
        return result;
    }
    
    public int getHeading ()
    {
        int result = 0;
        result = heading;
        return result;
    }
    
    public String toString ()
    {
        String result = "";
        result = result + "TurtleScreen : (x, y) = (" + x1 + ", " + y1 + ") : ";
        result = result + "heading = " + heading;
        return result;
    }
}
