package Prog1Tools;

import net.sourceforge.c4j.*;
/** 
 * Das Interface TurtleSpec spezifiziert die Methoden
 * zum Setzen und Bewegen einer zeichnenden Schildkroete (engl.: Turtle)
 * auf einem Bildschirm.
 * 
 * @author  hagen.buchwald@kit.edu
 * @version 1.0
 * @date 2010-11-14
 */
@ContractReference (contractClassName = "TurtleSpecContract")
public interface TurtleSpec
{
    /**
     * Kommando zum Setzen der Position der Schildkroete.
     * 
     * @param xPos x-Koordinate der Position der Schildkroete.
     * @param yPos y-Koordinate der Position der Schildkroete.
     */
    public void setTurtle (int xPos, int yPos);
    
    /**
     * Kommando zum Setzen der Richtung, in die die Schildkroete schaut.
     * 
     * @param heading Richtung in Grad, in die die Schildkroete schaut.
     *                0 Grad entspricht Blick nach Osten.
     *                90 Grad entspricht Blick nach Sden.
     *                180 Grad entspricht Blick nach Westen.
     *                270 Grad entspricht Blick nach Norden.
     *                Werte, die kleiner als 0 bzw. grer als 360 Grad sind,
     *                werden automatisch auf den Wertebereich von 0 bis 360 Grad
     *                umgerechnet.
     */
    public void setHeading (int heading);
    
    /**
     * Kommando zum Bewegen der Schildkroete um distance Einheiten in Blickrichtung.
     * 
     * @param distance Distanz, um die sich die Schildkroete in Blickrichtung bewegen soll.
     */
    public void forward (int distance);
    
    /**
     * Abfrage der x-Koordinate der Position der Schildkroete.
     * 
     * @return x-Koordinate der Position der Schildkroete.
     */
    @Pure
    public int getPositionX ();
    
    /**
     * Abfrage der y-Koordinate der Position der Schildkroete.
     * 
     * @return y-Koordinate der Position der Schildkroete.
     */
    @Pure
    public int getPositionY ();
    
    /**
     * Abfrage der Blickrichtung der Schildkroete.
     * 
     * @return Blickrichtung in Grad der Schildkroete.
     *         0 Grad entspricht Blick nach Osten.
     *         90 Grad entspricht Blick nach Sden.
     *         180 Grad entspricht Blick nach Westen.
     *         270 Grad entspricht Blick nach Norden.
     */
    @Pure
    public int getHeading ();

    /**
     * Kommando zum Einblenden der Schildkroete.
     */
    public void showTurtle ();

    /**
     * Kommando zum Ausblenden der Schildkroete.
     */
    public void hideTurtle ();

    /**
     * Abfrage, ob die Schildkrte eingeblendet ist.
     */
    @Pure
    public boolean isVisibleTurtle ();

    
}
    
